import java.util.*;

public class Generator {

  public static void main(String args[]) {
    Generator gen = new Generator();
    if( args.length > 0 ) { gen.generate(args[0]); }
    else { gen.generate(gen.read()); }
  }

  int apply[][][] = {
    { { 0, 2 }, { 2, 2 }, { 1, 2 } },
    { { 1, 2 }, { 0, 0 }, { 2, 1 } },
    { { 2, 2 }, { 1, 1 }, { 0, 0 } }
  };

  private final String prefix = "11021210112101221";

  class Circuit {
    int pos;
    int type;
    int self[];
    Circuit(int pos, int type) {
      this.pos = pos;
      this.type = type;
      self = new int[2];
    }
    int size() { return Math.min(2, type + 1); }
    int update(int req) {
      if( type == 0 ) {
        for(int i=0; i<3; ++i) {
          int res[] = apply[i][self[1]];
          if( res[0] == req ) {
            self[1] = res[1];
            return i;
          }
        }
      }
      if( type == 1 ) {
        int res[] = apply[self[0]][self[1]];
        self[0] = res[0];
        for(int i=0; i<3; ++i) {
          int res2[] = apply[i][res[1]];
          if( res2[0] == req ) {
            self[1] = res2[1];
            return i;
          }
        }
      }
      if( type == 2 ) {
        int res[] = apply[self[0]][self[1]];
        self[0] = res[0];
        for(int i=0; i<3; ++i) {
          int res2[] = apply[res[1]][i];
          if( res2[0] == req ) {
            self[1] = res2[1];
            return i;
          }
        }
      }
      throw new Error();
    }
    String joint(String in, String out) {
      if( type == 0 ) {
        return in + pos + "R0#" + out + pos + "R";
      }
      if( type == 1 ) {
        return (pos + "L" + (pos + 1) + "R0#" + pos + "L" + (pos + 1) + "R,\n" +
                in + pos + "R0#" + out + pos + "R");
      }
      if( type == 2 ) {
        return (pos + "L" + (pos + 1) + "R0#" + pos + "L" + (pos + 1) + "L,\n" +
                pos + "R" + in + "0#" + out + pos + "R");
      }
      return null;
    }
    String in() { return (pos + size() - 1) + (type == 2 ? "R" : "L"); }
    String out() { return (pos + size() - 1) + "L"; }
  }

  void generate(String s) {
    s = prefix + s;
    int len = s.length();
    Circuit cir[] = new Circuit[len];
    int gates[] = new int[len];
    int state[] = new int[len];
    int pos = 0;;
    for(int i=0; i<len; ++i) {
      int req = s.charAt(i) - '0';
      for(int j=0; j<i; ++j) { req = cir[j].update(req); }
      cir[i] = new Circuit(pos, req);
      cir[i].update(req);
      pos += cir[i].size();
    }
    System.out.println(cir[len-1].in() + ":");
    for(int i=0; i<len; ++i) {
      String in = (i + 1 == len ? "X" : cir[i+1].out());
      String out = (i == 0 ? "X" : cir[i-1].in());
      System.out.println(cir[i].joint(in, out) + (i + 1 == len ? ":" : ","));
    }
    System.out.println(cir[0].out());
  }

  String read() {
    Scanner cin = new Scanner(System.in);
    return cin.next();
  }

}
