//The MIT License
//
//Copyright (c) 2009 nodchip
//
//Permission is hereby granted, free of charge, to any person obtaining a copy
//of this software and associated documentation files (the "Software"), to deal
//in the Software without restriction, including without limitation the rights
//to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
//copies of the Software, and to permit persons to whom the Software is
//furnished to do so, subject to the following conditions:
//
//The above copyright notice and this permission notice shall be included in
//all copies or substantial portions of the Software.
//
//THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
//IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
//FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
//AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
//LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
//OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
//THE SOFTWARE.

#include <iostream>
#include <fstream>
#include <vector>
#include <cstdlib>
#include <ctime>
#include <cmath>
#include <limits>

using namespace std;

#include "calc_tmp1.cc"
/*const int MNUM = 2;
  const int MSIZE = 7; */


//TODO:状態を表す型/構造体を作成する
//typedef hoge STATE

struct Matrix {
    vector<vector<double> > A;

    Matrix() : A(MSIZE, vector<double>(MSIZE)) {
    }

    double& operator()(int i, int j) {
        return A[i][j];
    }

    double operator()(int i, int j) const {
        return A[i][j];
    }


    friend Matrix operator*(const Matrix& a, const Matrix& b) {
        Matrix m;
        for (int i = 0; i < MSIZE; ++i) {
            for (int j = 0; j < MSIZE; ++j) {
                for (int k = 0; k < MSIZE; ++k) {
                    m(i, j) += a(i, k) * b(k, j);
                }
            }
        }

        return m;
    }
};

struct STATE
{
    Matrix A[MNUM];

    void print_for_ocaml(ostream& cout) {
        cout << "let fuel = [|" << endl;
        for (int k = 0; k < MNUM; ++k) {
            cout << "  [|" << endl;
            for (int i = 0; i < MSIZE; ++i) {
                cout << "    [|";
                for (int j = 0; j < MSIZE; ++j) {
                    cout << A[k](i, j);
                    if (j + 1 != MSIZE) { cout << ';'; }
                }
                cout << "|]";
                if (i + 1 != MSIZE) { cout << ";" << endl; } else { cout << endl; }
            }
            cout << "  |]" << endl;
            if (k + 1 != MNUM) { cout <<  ";" << endl; } else { cout << endl; }
        }
        cout << "|];;" << endl;
    }
};

class SimulatedAnnealing
{
public:
	//TODO:コンストラクタに必要な引数を追加する
	SimulatedAnnealing();
	STATE run();
public:
	static const int TIME_LIMIT;
	double calculateProbability(double score, double scoreNeighbor, double temperature);
	STATE createInitialState();
	STATE createNeighborState(const STATE& currentState);
	double calculateEnergy(const STATE& state);
	//TODO:以下に内部変数を追加する
};

//TODO:計算時間をミリ秒単位で設定する
const int SimulatedAnnealing::TIME_LIMIT = 10000;

STATE SimulatedAnnealing::run()
{
	STATE state = createInitialState();
	double energy = calculateEnergy(state);
	STATE result = state;
	double minEnergy = energy;
	const int clockStart = (int)clock();
	const int clockEnd = clockStart + TIME_LIMIT * (CLOCKS_PER_SEC / 1000);
	int counter = 0;
	int clockCurrent;
	while ((clockCurrent = clock()) < clockEnd){
		const STATE stateNeighbor = createNeighborState(state);
		const double energyNeighbor = calculateEnergy(stateNeighbor);
		const double random = rand() / (double)RAND_MAX;
		const double probability = calculateProbability(energy, energyNeighbor, (double)(clockEnd - clockCurrent) / (double)(clockEnd - clockStart) + 1);
		if (random < probability){
			state = stateNeighbor;
			energy  = energyNeighbor;
			if (minEnergy > energy){
				result = state;
				minEnergy = energy;
				//fprintf(stderr, "%.5lf\n", minEnergy);
			}
		}
		++counter;
	}
	//fprintf(stderr, "%d\n", counter);
	return result;
}

double SimulatedAnnealing::calculateProbability(double energy, double energyNeighbor, double temperature)
{
	return energyNeighbor < energy ? 1 : exp((energy - energyNeighbor) / temperature);
}

//TODO:コンストラクタを記述する
SimulatedAnnealing::SimulatedAnnealing()
{
}

//TODO:初期状態を作る関数を記述する
STATE SimulatedAnnealing::createInitialState()
{
    STATE st;

    for (int k = 0; k < MNUM; ++k) {
        for (int i = 0; i < MSIZE; ++i) {
            for (int j = 0; j < MSIZE; ++j) {
                st.A[k](i, j) = rand() % 100 + 1;
            }
        }
    }

    return st;
}

//TODO:遷移後の状態を作る関数を記述する
STATE SimulatedAnnealing::createNeighborState(const STATE& currentState)
{
    STATE st(currentState);

    if ((rand() >> 4) & 1) { // big step
        int cnt = (MNUM + MSIZE) / 2 + 1;
        while (cnt--) {
            int r = rand() % MNUM;
            int i = rand() % MSIZE;
            int j = rand() % MSIZE;
            int v = rand() % 100;
            
            if (st.A[r](i, j) + (v - 50) >= 0) {
                st.A[r](i, j) += (v - 50);
            }            
        }

        return st;
    } else { // small step
        while (true) {
            int r = rand() % MNUM;
            int i = rand() % MSIZE;
            int j = rand() % MSIZE;
            int v = rand() % 10;
            
            if (st.A[r](i, j) + (v - 5) >= 0) {
                st.A[r](i, j) += (v - 5);
                return st;
            }
        }
    }
}

double P(const Matrix& A, const Matrix& B)
{
    double ans = 0;
    for (int i = 0; i < MSIZE; ++i) {
        for (int j = 0; j < MSIZE; ++j) {
            if (A(i, j) < B(i, j)) {
                ans += (B(i, j) - A(i, j));
            }
        }
    }
    return ans;
}

double Q(const Matrix& A, const Matrix& B)
{
    double ans = 0;
    for (int i = 0; i < MSIZE; ++i) {
        for (int j = 0; j < MSIZE; ++j) {
            if (A(i, j) <= B(i, j)) {
                ans += (B(i, j) - A(i, j));
                ans += 10;
            }
        }
    }
    return ans;
}

//TODO:状態のエネルギーを計算する関数を記述する
//(状態はエネルギーの低い所に遷移する点に注意する)
double SimulatedAnnealing::calculateEnergy(const STATE& state)
{
    double ans = 0;

    // consistency check
    for (int k = 0; k < MNUM; ++k) {
        if (state.A[k](0, 0) <= 0) { return numeric_limits<double>::infinity(); }
        for (int i = 0; i < MSIZE; ++i) {
            for (int j = 0; j < MSIZE; ++j) {
                if (state.A[k](i, j) < 0) { return numeric_limits<double>::infinity(); }
            }
        }
    }

#include "calc_tmp2.cc"

    if (ans > 0) { return ans; }

    // if solved, we want to use more efficient answer.
    for (int k = 0; k < MNUM; ++k) {
        for (int i = 0; i < MSIZE; ++i) {
            for (int j = 0; j < MSIZE; ++j) {
                ans -= 1.0 / state.A[k](i, j);
            }
        }
    }
    
    return ans;
}

int main()
{
    srand(unsigned(time(NULL)));

    SimulatedAnnealing sa;
    STATE st = sa.run();

    double energy = sa.calculateEnergy(st);
    if (energy <= 0) {
        cerr << "enegy is non-positive! solved!!!";
        ofstream fs("fuel.ml");
        st.print_for_ocaml(cerr);
        st.print_for_ocaml(fs);
        return 0;
    } else {
        cerr << "energy is positive (" << energy << "). could not be solved." << endl;
        st.print_for_ocaml(cerr);
        return -1;
    }
}
