import std.stdio;
import std.file;
import std.typecons;
import std.string;
import std.conv;
import std.algorithm;
import std.system;

enum { L = 0, R = 1 };
alias Tuple!(int, "p", int, "lr") Port;

class Parser
{
private:
	string s;

	Port read_port()
	{
		if( s[0] == 'X' ) {
			s = s[1..$];
			return Port(-1, L);
		}
		int p = to!int( s.munch("0123456789") );
		int lr = (s.munch("LR")[0]=='L' ? L : R);
		return Port(p,lr);
	}

	void skip_ws()
	{
		s.munch(" \t\r\n");
	}

	char sep()
	{
		char c = s[0];
		s = s[1..$];
		skip_ws();
		return c;
	}

	void gate()
	{
		s = s[2..$];
	}

public:
	this(string s){this.s = s;}

	Circuit read_circuit()
	{
		Circuit cc = new Circuit;

		cc.pi = read_port();
		sep();
		for(int g=0 ;; ++g) {
			Port p1 = read_port();
			Port p2 = read_port();
			gate();
			Port p3 = read_port();
			Port p4 = read_port();
			char c = sep();
			cc.add(p1,p2,p3,p4);
			if( c == ':' )
				break;
		}
		cc.po = read_port();

		return cc;
	}
}

class Circuit
{
	Port pi, po;
	Port[] inL, inR, outL, outR;
	void add(Port p1, Port p2, Port p3, Port p4)
	{
		inL ~= p1;
		inR ~= p2;
		outL ~= p3;
		outR ~= p4;
	}
	int gateNum() { return inL.length; }

	Executer create_executer() { return new Executer; }

	class Executer
	{
	private:
		int      input_gate;
		int[2][] gate;
		int read(Port p)
		{
			return p.p == -1 ? input_gate : gate[p.p][p.lr];
		}
		this() { gate = new int[2][gateNum()]; }

	public:
		int run(int X)
		{
			input_gate = X;
			for(int g=0, ge=gateNum(); g<ge; ++g)
			{
				int a = read(inL[g]);
				int b = read(inR[g]);
				gate[g][L] = (a-b+3) % 3;
				gate[g][R] = (a*b+2) % 3;
			}
			return gate[po.p][po.lr];
		}

		int[] run(int[] X)
		{
			int[] result;
			foreach(x; X)
				result ~= run(x);
			return result;
		}

		int[] run_with_const_input(int n)
		{
			int[] result;
			foreach(x; 0..n)
				result ~= run(0);
			return result;
		}
	}
}

string const_gen(int a, int b, string inP, string outP, bool isend=false)
{
	string result;
	for(int i=a; i<=b; ++i)
	{
		bool cross = (i-a)%2==0 || ((i-a)%8==7 && i!=b);
		int j = (i==a ? b : i-1);
		bool precross = (j-a)%2==0 || ((j-a)%8==7 && j!=b);

		if(!precross)
			result ~=
				(i==a ? inP : to!string(i-1)~"L") ~
				(to!string(i==a ? b : i-1)~"R");
		else
			result ~=
				(to!string(i==a ? b : i-1)~"R") ~
				(i==a ? inP : to!string(i-1)~"L");
		result ~= "0#";
		if( !cross )
			result ~=
				(i==b ? outP : to!string(i+1)~"L")~
				(to!string(i==b ? a : i+1)~"R");
		else
			result ~=
				(to!string(i==b ? a : i+1)~"R") ~
				(i==b ? outP : to!string(i+1)~"L");
		result ~=
			(isend&&i==b ? ":\n" : ",\n");
	}
	return result;
}

string generate_const_one()
{
	string result;
	result = "0L:\n";
	result ~= const_gen( 0, 23, "X", "X", true );
	result ~= "23L\n";
	return result;
}

string genblk(int BEG, int END, string a, string b, string c, string d, bool isend=false )
{
	string result;
	for(int n=BEG; n<=END; ++n)
	{
		result ~= (n==BEG ? a : to!string(n-1)~"L");
		result ~= (n==END ? b : to!string(n==BEG ? END-1 : n-1)~"R");
		result ~= "0#";
		result ~= (n==END ? c : to!string(n+1)~"L");
		result ~= (n==END ? d : to!string(n==END-1 ? BEG : n+1)~"R");
		result ~= (isend && n==END ? ":\n" : ",\n");
	}
	return result;
}

version = chess;

string generate( int[] numBlks )
{
	assert( numBlks.length >= 3 );

	string result;

version(alwaysconst)
{
	result = "0L:\n";
	result ~= const_gen( 0, 23, "X", "24L" );
	int start = 24;
	string const_last = "23L";
}
else
version(gekitsui)
{
result = `0R:
0LX0#0L1L,
0R4L0#2L2R,
1L1R0#3L3R,
2L2R0#4L4R,
3L3R0#1R5L,
`;
	int start = 5;
	string const_last = "4R";
}
else
version(chess)
{
result = `0L:
X5L0#1R1L,
0R0L0#2L2R,
1L1R0#3L3R,
2L2R0#4R4L,
3R3L0#5L5R,
4L4R0#0R6L,
`;
	int start = 6;
	string const_last = "5R";
}

	int[] beg, end;
	foreach(n; numBlks) {
		beg ~= start;
		end ~= start+n-1;
		start += n;
	}

	result ~= genblk( beg[0], end[0],
			const_last, to!string(end[1])~"L",
			"X",   to!string(beg[1])~"L"
	);
	for(int i=1; i<numBlks.length-1; ++i)
		result ~= genblk( beg[i], end[i],
			to!string(end[i-1])~"R", to!string(end[i+1])~"L",
			to!string(end[i-1])~"R", to!string(beg[i+1])~"L"
		);
	int i = numBlks.length-1;
	result ~= genblk( beg[i], end[i],
		to!string(end[i-1])~"R", to!string(end[i])~"R",
		to!string(end[i-1])~"R", to!string(end[i])~"R",
		true
	);
	result ~= to!string(end[0])~"L";
	return result;
}

string search(string expected_output)
{
	int[] a;
	foreach(c; expected_output)
		a ~= c-'0';
	return search(a);
}

string search(int[] expected_output)
{
	assert( expected_output.length >= 3 );

	int[] blk;
	foreach(_; expected_output) blk ~= 1;

	for(int i=0; i<expected_output.length; ++i)
	{
		for(;;) {
			string b = generate(blk);

			auto cir = (new Parser(b)).read_circuit();
			auto exe = cir.create_executer();
			int[] res = exe.run_with_const_input(i+1);
			if( res == expected_output[0..i+1] ) {
				if( i == expected_output.length-1 )
					return b;
				else
					break;
			}
			blk[i] ++;
		}
	}

	return null;
}


void simulator_main(string[] argv)
{
	string s;
	if( argv.length <= 1 )
		s = std.stdio.readln('\0');
	else
		s = std.file.readText(argv[1]);

	auto cir = (new Parser(s)).read_circuit();
	auto exe = cir.create_executer();

	int[] inp = [0,2,2,2,2,2,2,0,2,1,0,1,1,0,0,1,1];
	int[] oup = exe.run(inp);
	writeln(oup);
}

void searcher_main(string[] argv)
{
	if( argv.length <= 1 || argv[1]=="-r" && argv.length<=2 ) {
		try {
			for(;;) {
				string s = readln();
				s = s.chomp();
				if( s.length == 0 )
					break;
				if( argv.length<=1 )
					writeln(search("11021210112101221"~s));
				else
					writeln(search(s));
			}
		} catch {}
	}
	else if( argv[1]=="-r" )
		writeln(search(argv[2]));
	else
		writeln(search("11021210112101221"~argv[1]));
}

void main(string[] argv)
{
	searcher_main(argv);
}
