#include <iostream>
#include <Eigen/StdVector>
#include <Eigen/Core>
#include <vector>
#include <algorithm>
#include <ctime>
#include <cstdlib>
#include <string>
#include <fstream>
using namespace std;
typedef long long LL;

USING_PART_OF_NAMESPACE_EIGEN

static const int DIM = 6;
static const int TANK_NUM = 6;
static const int EXPR_NUM = 10000;
static const int CHAM_NUM = 200;

typedef Matrix<LL, DIM, DIM> MAT;
typedef vector<int> EXPR;
enum TYPE { AsMain, AsAux, Bad };

MAT gen_matrix()
{
	MAT m;
	for(int i=0; i<DIM; ++i)
	for(int j=0; j<DIM; ++j)
		m(i,j) = 1 + rand() % 10;
	return m;
}

MAT gen_matrix_swapper()
{
	int idx[DIM];
	for(int i=0; i<DIM; ++i) idx[i]=i;
	random_shuffle(&idx[0], &idx[DIM]);

	MAT m;
	for(int i=0; i<DIM; ++i)
		for(int j=0; j<DIM; ++j)
			m(i,j) = (j==idx[i] ? 1 : 0);
	m(0,0) = 1;
	return m;
}

vector<int> gen_expr()
{
	vector<int> v;
	int n = 10 + rand()%4 - 2;
	if(n==0)n=2;
	for(int i=0; i<n; ++i)
		v.push_back( rand()%TANK_NUM );
	return v;
}

MAT eval( vector<int>& e, vector<MAT>& t )
{
	MAT m = MAT::Identity();
	for(int i=0; i<e.size(); ++i)
		m *= t[e[i]];
	return m;
}

LL judge( MAT& lhs, MAT& rhs )
{
	int t = 0;
	LL sumdif = 0;
	for(int i=0; i<DIM; ++i)
		for(int j=0; j<DIM; ++j) {
			if( lhs(i,j) < rhs(i,j) )
				return -2;
			if( i==0 && lhs(i,j)==rhs(i,j) )
				t = -1;
			sumdif = (lhs(i,j) - rhs(i,j));
		}
	if(t<0)return t;
	return sumdif;
}

bool my_less(int a, int b)
{
	return a<b;
}

template<typename T>
bool my_less( vector<T>& a, vector<T>& b )
{
	if( a.size() < b.size() ) return true;
	if( a.size() > b.size() ) return false;
	for(int i=0; i<a.size(); ++i) {
		if( my_less(a[i], b[i]) ) return true;
		if( my_less(b[i], a[i]) ) return false;
	}
	return false;
}

void minimize( vector< vector<int> >& lhs, vector< vector<int> >& rhs )
{
	vector<int> idx;
	for(int i=0; i<lhs.size(); ++i) idx.push_back(i);
	sort(idx.begin(), idx.end(), [&](int a, int b) -> bool {
		if( my_less(lhs[a], lhs[b]) ) return true;
		if( my_less(lhs[b], lhs[a]) ) return false;
		if( my_less(rhs[a], rhs[b]) ) return true;
		if( my_less(rhs[b], rhs[a]) ) return false;
		return false;
	});

	vector< vector<int> > lhs2(lhs.size());
	for(int i=0; i<lhs.size(); ++i)
		lhs2[i] = lhs[idx[i]];
	lhs = lhs2;
	vector< vector<int> > rhs2(rhs.size());
	for(int i=0; i<rhs.size(); ++i)
		rhs2[i] = rhs[idx[i]];
	rhs = rhs2;
}



void lexLeast( vector< vector<int> >& lhs, vector< vector<int> >& rhs, vector<MAT>& t )
{
	vector< vector<int> > lhs_min;
	vector< vector<int> > rhs_min;
	vector<MAT> t_min;
	bool first = true;

	int idx[] = {0,1,2,3,4,5};
	do {
		vector<MAT> t2(TANK_NUM);
		for(int i=0; i<TANK_NUM; ++i)
			t2[idx[i]] = t[i];

		vector< vector<int> > lhs2 = lhs;
		vector< vector<int> > rhs2 = rhs;
		for(int i=0; i<lhs.size(); ++i) {
			for(int j=0; j<lhs[i].size(); ++j)
				lhs2[i][j] = idx[lhs[i][j]];
			for(int j=0; j<rhs[i].size(); ++j)
				rhs2[i][j] = idx[rhs[i][j]];
		}

		minimize(lhs2, rhs2);
		if( first )
			{lhs_min=lhs2; rhs_min=rhs2; t_min=t2;}
		else {
			bool ok = false;
			for(int i=0; i<lhs2.size(); ++i) {
				if( my_less(lhs2[i],lhs_min[i]) ) {ok = true; break;}
				if( my_less(lhs_min[i],lhs2[i]) ) {ok = false; break;}
				if( my_less(rhs2[i],rhs_min[i]) ) {ok = true; break;}
				if( my_less(rhs_min[i],rhs2[i]) ) {ok = false; break;}
			}
			if(ok)
				{lhs_min=lhs2; rhs_min=rhs2; t_min=t2;}
		}
		first = false;
	} while (next_permutation(&idx[0], &idx[TANK_NUM]));

	lhs = lhs_min;
	rhs = rhs_min;
	t = t_min;
}

int main()
{
	srand(time(0));

	vector<MAT> t;
	for(int i=0; i<2; ++i)
		t.push_back( gen_matrix() );
	for(int i=2; i<TANK_NUM; ++i)
		t.push_back( gen_matrix_swapper() );

	vector< vector<int> > e;
	for(int i=0; i<EXPR_NUM; ++i)
		e.push_back( gen_expr() );
	e.erase(unique(e.begin(), e.end()),e.end());

	vector<bool> used_rhs(e.size());

	vector<EXPR> lhs, rhs;
	vector<LL> score;
	for(int i=0; i<e.size(); ++i)
	{
		MAT ll = eval( e[i], t );

		for(int j=e.size()-1; j>=0; --j)
			if( i != j && !used_rhs[j] )
			{
				MAT rr = eval( e[j], t );
				LL jg = judge(ll, rr);
				if( jg >= 0 ) {
					lhs.push_back(e[i]);
					rhs.push_back(e[j]);
					score.push_back(jg);
					used_rhs[j] = true;
					if( lhs.size() >= CHAM_NUM*10 ) {
						goto found;
					}
					goto nextI;
				}
			}
	nextI:;
	}

found:

	vector<int> iii(lhs.size());
	for(int i=0; i<lhs.size(); ++i) iii.push_back(i);
	sort(iii.begin(), iii.end(), [&](int a, int b)->bool {
		return score[a] < score[b];
	});
	vector<EXPR> lhs2, rhs2;
	for(int i=0; i<lhs.size(); ++i) {
		lhs2.push_back( lhs[iii[i]] );
		rhs2.push_back( rhs[iii[i]] );
	}
	lhs = lhs2;
	rhs = rhs2;
	lhs.resize(CHAM_NUM);
	rhs.resize(CHAM_NUM);


	lexLeast(lhs, rhs, t);



	for(int i=0; i<lhs.size(); ++i) {
		MAT a = eval(lhs[i],t);
		MAT b = eval(rhs[i],t);
		assert( judge(a,b) >= 0 );
//		cerr<<a<<endl;
//		cerr<<b<<endl;
//		cerr<<endl;
	}

	ofstream ocaml("gen_car.ml");
	ocaml << "#use \"tencode.ml\""<<endl;
	ocaml << "let car = " << endl;
	ocaml << "[|" << endl;
	for(int i=0; i<lhs.size(); ++i)
	{
		ocaml << "{" << endl;
		ocaml << "upper = [|";
		for(int k=0; k<lhs[i].size(); ++k)
			ocaml << lhs[i][k] << (k==lhs[i].size()-1 ? "" : ";");
		ocaml << "|];" << endl;
		ocaml << "main = true;" << endl;
		ocaml << "lower = [|";
		for(int k=0; k<rhs[i].size(); ++k)
			ocaml << rhs[i][k] << (k==rhs[i].size()-1 ? "" : ";");
		ocaml << "|]" << endl;
		ocaml << "}";
		if( i!=lhs.size()-1 ) ocaml << ";" ;
		ocaml << endl;
	}
	ocaml << "|]" << endl;
	ocaml << ";;" << endl;
	ocaml << "let _ = print_car car;;"<<endl;


	ofstream fuel("gen_fuel.ml");
	fuel << "#use \"tencode.ml\""<<endl;
	fuel << "let fuel = " << endl;
	fuel << "[|";
	for(int i=0; i<t.size(); ++i)
	{
		fuel << "[|";
		for(int j=0; j<DIM; ++j)
		{
			fuel << "[|";
			for(int k=0; k<DIM; ++k)
				fuel << t[i](j,k) << (k==DIM-1 ? "" : ";");
			fuel << "|]" << (j==DIM-1 ? "" : ";");
		}
		fuel << "|]" << (i==t.size()-1 ? "" : ";");
	}
	fuel << "|];;" <<endl;
	fuel << "let _ = print_fuel fuel;;"<<endl;
}
