(* decode terminary streams  *)

let rec pow x y =
  if y == 0 then 1
  else x * pow x (pred y)
;;

let rec parse_int str rpos = 
  match str.[!rpos] with
    | '0' ->
        incr rpos; 0
    | '1' -> 
        incr rpos;
        let answer = 1 + (Char.code str.[!rpos] - Char.code '0') in
        incr rpos;
        answer
    | '2' ->
        incr rpos; assert (str.[!rpos] = '2'); incr rpos;
        let x = parse_int str rpos + 2 in
(*        Printf.printf "x = %d : rpos = %d\n" x !rpos; *)
        let alpha = 
          let rec loop sum i = 
            if i < x then
              loop (sum + pow 3 i) (succ i)
            else 
              sum
          in loop 0 0
        in
        let tt = String.sub str !rpos x in
(*        print_string tt; *)
        let beta = 
          let rec loop sum i =
            if i < String.length tt then
              loop (sum * 3 + (Char.code tt.[i] - Char.code '0')) (succ i)
            else
              sum
          in loop 0 0
        in
        rpos := !rpos + x;
        let answer = alpha + beta in
(*         Printf.printf "%d %d %d" alpha beta answer; *)
        answer
    | _ -> failwith "unknown token."
;;

(* unit test *)
let _ = begin
  assert (parse_int "0" (ref 0) = 0);
  assert (parse_int "10" (ref 0) = 1);
  assert (parse_int "11" (ref 0) = 2);
  assert (parse_int "12" (ref 0) = 3);
  assert (parse_int "22000" (ref 0) = 4);
  assert (parse_int "22001" (ref 0) = 5);
  assert (parse_int "22002" (ref 0) = 6);
  assert (parse_int "22010" (ref 0) = 7);
  assert (parse_int "22011" (ref 0) = 8);
  assert (parse_int "22012" (ref 0) = 9);
  assert (parse_int "22020" (ref 0) = 10);
  assert (parse_int "22021" (ref 0) = 11);
  assert (parse_int "22022" (ref 0) = 12);
  assert (parse_int "2210000" (ref 0) = 13);
  assert (parse_int "2210222" (ref 0) = 39);
  assert (parse_int "22110000" (ref 0) = 40);
  assert (parse_int "22112222" (ref 0) = 120);
  assert (parse_int "221200000" (ref 0) = 121);
  assert (parse_int "221222222" (ref 0) = 363);
  assert (parse_int "2222000000000" (ref 0) = 364);

  print_endline "OK";
end

let parse_list_num str rpos: int =
  match str.[!rpos] with
    | '0' -> incr rpos; 0
    | '1' -> incr rpos; 1
    | '2' ->
        incr rpos; assert (str.[!rpos] == '2'); incr rpos;
        2 + parse_int str rpos
    | _ -> failwith "unknown"
;;

(* unit test *)
let _ = begin
  assert (parse_list_num "0" (ref 0)    = 0);
  assert (parse_list_num "000" (ref 0)    = 0);
  assert (parse_list_num "011" (ref 0)    = 0);
  assert (parse_list_num "1" (ref 0)    = 1);
  assert (parse_list_num "220" (ref 0)  = 2);
  assert (parse_list_num "2210" (ref 0) = 3);
  assert (parse_list_num "2211" (ref 0) = 4);
  assert (parse_list_num "2212" (ref 0) = 5);
  assert (parse_list_num "2222000" (ref 0) = 6);
end

let parse_list str rpos =
  let n: int = parse_list_num str rpos in
  let rec loop i =
    if i < n then begin
      let t = parse_int str rpos in
      Printf.printf "[%d]" t;
      loop (succ i)
    end
  in loop 0
;;

let parse_main_or_aux str rpos = 
  match parse_int str rpos with
    | 0 -> print_string "yes"
    | 1 -> print_string "no"
    | _ -> failwith "parsing error: main or aux"
;;

let _ = 
  while true do
    let str = read_line ()
    and rpos = ref 0 in
    let n = parse_list_num str rpos in
    Printf.printf "******** Chamber num: %d\n" n;
    let rec loop i = 
      if i < n then begin
        Printf.printf "chamber #%d\n" i;
        Printf.printf "upper: ";
        parse_list str rpos; print_newline ();
        Printf.printf "main?: ";
        parse_main_or_aux str rpos; print_newline ();
        Printf.printf "lower: ";
        parse_list str rpos; print_newline ();
        print_newline ();
        loop (succ i)
      end
    in loop 0
  done
;;

