
type chamber = {
  upper: int array;
  main:  bool;
  lower: int array;
}

type car = chamber array

let car = [|
  { upper = [|0;0|];
    main = true;
    lower = [|0|] }
|]

let fuel = [|
  [| 
    [|3; 57; 1; 18; 29; 352; 94 |];
    [|0; 0; 0; 0; 19; 307; 72 |];
    [|1;1;0;0;45;209;501 |];
    [|8;1;1;1;0;15;14 |];
    [|0;0;0;1;1;0;0 |];
    [|0;1;0;0;7;131;586 |];
    [|0;36;0;0;0;0;0 |]
  |];
  [|
    [|1;0;0;0;0;0;42 |];
    [|0;0;0;0;1;0;67 |];
    [|0;0;0;0;0;0;77 |];
    [|0;0;0;1;1;0;68 |];
    [|0;0;0;0;0;0;1 |];
    [|0;0;0;0;0;0;0 |];
    [|0;0;0;0;0;0;0 |]
  |]
|];;

(* ---------------------------------------------------------------------- *)

let rec pow x y =
  if y == 0 then 1
  else x * pow x (pred y)
;;

let encode_fill n len = 
  let str = String.make len '0' in
  let v = ref n in
  for i = 0 to len - 1 do 
    str.[len - i - 1] <- Char.chr ((!v mod 3) + Char.code '0');
    v := !v / 3;
  done;
  str
;;

(* unit test *)
let _ = begin
  assert (encode_fill 0 1 = "0");
  assert (encode_fill 1 2 = "01");
  assert (encode_fill 2 3 = "002");
  assert (encode_fill 3 4 = "0010");
end;;

let rec encode_int n =
  let rec encode_int_sub n i =
    if n < pow 3 i then begin
      (encode_size i) ^ (encode_fill n i)
    end else begin
      encode_int_sub (n - pow 3 i) (succ i)
    end
  in
  encode_int_sub n 0
and encode_size n =
  match n with
    | 0 -> "0"
    | 1 -> "1"
    | _ -> "22" ^ (encode_int (n - 2))
;;

(* unit test *)
let _ = begin
  assert (encode_int 0 = "0");
  assert (encode_int 1 = "10");
  assert (encode_int 2 = "11");
  assert (encode_int 3 = "12");
  assert (encode_int 4 = "22000");
  assert (encode_int 5 = "22001");
  assert (encode_int 6 = "22002");
  assert (encode_int 7 = "22010");
  assert (encode_int 8 = "22011");
  assert (encode_int 9 = "22012");
  assert (encode_int 10 = "22020");
  assert (encode_int 11 = "22021");
  assert (encode_int 12 = "22022");
  assert (encode_int 13 = "2210000");

  assert (encode_size 0 = "0");
  assert (encode_size 1 = "1");
  assert (encode_size 2 = "220");
  assert (encode_size 3 = "2210");
  assert (encode_size 4 = "2211");
  assert (encode_size 5 = "2212");
  assert (encode_size 6 = "2222000");
  prerr_endline "OK"
end


let print_car car = 
  let n = Array.length car in
  print_string (encode_size n);
  Array.iter (fun chamber ->
    print_string (encode_size (Array.length chamber.upper));
    Array.iter (fun u -> print_string (encode_int u)) chamber.upper;
    print_string (if chamber.main then "0" else "10");
    print_string (encode_size (Array.length chamber.lower));
    Array.iter (fun u -> print_string (encode_int u)) chamber.lower;
  ) car;
  print_newline ()
;;

let print_fuel fuel = 
  let n = Array.length fuel in
  print_string (encode_size n);
  for k = 0 to n - 1 do
    let m = fuel.(k) in
    let s = Array.length m in
    print_string (encode_size s);
    for i = 0 to s - 1 do
      print_string (encode_size s);
      for j = 0 to s - 1 do
        print_string (encode_int fuel.(k).(i).(j))
      done;
    done;
  done;
  print_newline ()
;;

(*
let _ = print_fuel fuel 
*)
(*
let _ = 
  print_encode car
;;
*)



